@@ -1285,4 +1285,9 @@ public class BookieImpl extends BookieCriticalThread implements Bookie {
             }
         }
     }
+
+    @VisibleForTesting
+    public List<Journal> getJournals() {
+        return this.journals;
+    }
 }
