@@ -193,6 +193,11 @@ public class BookieStateManager implements StateManager {
         return forceReadOnly.get() || bookieStatus.isInReadOnlyMode();
     }
 
+    @Override
+    public boolean isForceReadOnly(){
+        return forceReadOnly.get();
+    }
+
     @Override
     public boolean isAvailableForHighPriorityWrites() {
         return availableForHighPriorityWrites;
