@@ -41,6 +41,7 @@ import org.apache.bookkeeper.auth.ClientAuthProvider;
 import org.apache.bookkeeper.client.BKException;
 import org.apache.bookkeeper.proto.BookkeeperProtocol.AuthMessage;
 import org.apache.bookkeeper.util.ByteBufList;
+import org.apache.bookkeeper.util.NettyChannelUtil;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -102,15 +103,15 @@ class AuthHandler {
             } else if (msg instanceof BookieProtocol.Request) {
                 BookieProtocol.Request req = (BookieProtocol.Request) msg;
                 if (req.getOpCode() == BookieProtocol.ADDENTRY) {
-                    ctx.channel().writeAndFlush(
-                            BookieProtocol.AddResponse.create(
-                                    req.getProtocolVersion(), BookieProtocol.EUA,
-                                    req.getLedgerId(), req.getEntryId()));
+                    final BookieProtocol.AddResponse response = BookieProtocol.AddResponse.create(
+                            req.getProtocolVersion(), BookieProtocol.EUA,
+                            req.getLedgerId(), req.getEntryId());
+                    NettyChannelUtil.writeAndFlushWithVoidPromise(ctx.channel(), response);
                 } else if (req.getOpCode() == BookieProtocol.READENTRY) {
-                    ctx.channel().writeAndFlush(
-                            new BookieProtocol.ReadResponse(
-                                    req.getProtocolVersion(), BookieProtocol.EUA,
-                                    req.getLedgerId(), req.getEntryId()));
+                    final BookieProtocol.ReadResponse response = new BookieProtocol.ReadResponse(
+                            req.getProtocolVersion(), BookieProtocol.EUA,
+                            req.getLedgerId(), req.getEntryId());
+                    NettyChannelUtil.writeAndFlushWithVoidPromise(ctx.channel(), response);
                 } else {
                     ctx.channel().close();
                 }
@@ -133,7 +134,7 @@ class AuthHandler {
                         .setHeader(req.getHeader())
                         .setStatus(BookkeeperProtocol.StatusCode.EUA);
 
-                    ctx.channel().writeAndFlush(builder.build());
+                    NettyChannelUtil.writeAndFlushWithVoidPromise(ctx.channel(), builder.build());
                 }
             } else {
                 // close the channel, junk coming over it
@@ -172,7 +173,9 @@ class AuthHandler {
                 }
                 AuthMessage message = AuthMessage.newBuilder().setAuthPluginName(req.authMessage.getAuthPluginName())
                         .setPayload(ByteString.copyFrom(newam.getData())).build();
-                channel.writeAndFlush(new BookieProtocol.AuthResponse(req.getProtocolVersion(), message));
+                final BookieProtocol.AuthResponse response =
+                        new BookieProtocol.AuthResponse(req.getProtocolVersion(), message);
+                NettyChannelUtil.writeAndFlushWithVoidPromise(channel, response);
             }
         }
 
@@ -196,14 +199,17 @@ class AuthHandler {
                     LOG.error("Error processing auth message, closing connection");
 
                     builder.setStatus(BookkeeperProtocol.StatusCode.EUA);
-                    channel.writeAndFlush(builder.build());
-                    channel.close();
+                    NettyChannelUtil.writeAndFlushWithClosePromise(
+                            channel, builder.build()
+                    );
                     return;
                 } else {
                     AuthMessage message = AuthMessage.newBuilder().setAuthPluginName(pluginName)
                             .setPayload(ByteString.copyFrom(newam.getData())).build();
                     builder.setStatus(BookkeeperProtocol.StatusCode.EOK).setAuthResponse(message);
-                    channel.writeAndFlush(builder.build());
+                    NettyChannelUtil.writeAndFlushWithVoidPromise(
+                            channel, builder.build()
+                    );
                 }
             }
         }
@@ -399,9 +405,9 @@ class AuthHandler {
                         .setPayload(ByteString.copyFrom(newam.getData())).build();
 
                 if (isUsingV2Protocol) {
-                    channel.writeAndFlush(
-                            new BookieProtocol.AuthRequest(BookieProtocol.CURRENT_PROTOCOL_VERSION, message),
-                            channel.voidPromise());
+                    final BookieProtocol.AuthRequest msg =
+                            new BookieProtocol.AuthRequest(BookieProtocol.CURRENT_PROTOCOL_VERSION, message);
+                    NettyChannelUtil.writeAndFlushWithVoidPromise(channel, msg);
                 } else {
                     // V3 protocol
                     BookkeeperProtocol.BKPacketHeader header = BookkeeperProtocol.BKPacketHeader.newBuilder()
@@ -410,7 +416,7 @@ class AuthHandler {
                     BookkeeperProtocol.Request.Builder builder = BookkeeperProtocol.Request.newBuilder()
                             .setHeader(header)
                             .setAuthRequest(message);
-                    channel.writeAndFlush(builder.build());
+                    NettyChannelUtil.writeAndFlushWithVoidPromise(channel, builder.build());
                 }
             }
         }
@@ -429,7 +435,7 @@ class AuthHandler {
                         authenticated = true;
                         Object msg = waitingForAuth.poll();
                         while (msg != null) {
-                            ctx.writeAndFlush(msg);
+                            NettyChannelUtil.writeAndFlushWithVoidPromise(ctx, msg);
                             msg = waitingForAuth.poll();
                         }
                     }
