@@ -391,27 +391,18 @@ public class Auditor implements AutoCloseable {
             try {
                 watchBookieChanges();
                 knownBookies = getAvailableBookies();
+                this.ledgerUnderreplicationManager
+                        .notifyLostBookieRecoveryDelayChanged(new LostBookieRecoveryDelayChangedCb());
+                this.ledgerUnderreplicationManager.notifyUnderReplicationLedgerChanged(
+                        new UnderReplicatedLedgersChangedCb());
             } catch (BKException bke) {
                 LOG.error("Couldn't get bookie list, so exiting", bke);
                 submitShutdownTask();
-            }
-
-            try {
-                this.ledgerUnderreplicationManager
-                        .notifyLostBookieRecoveryDelayChanged(new LostBookieRecoveryDelayChangedCb());
+                return;
             } catch (UnavailableException ue) {
-                LOG.error("Exception while registering for LostBookieRecoveryDelay change notification, so exiting",
-                        ue);
-                submitShutdownTask();
-            }
-
-            try {
-                this.ledgerUnderreplicationManager.notifyUnderReplicationLedgerChanged(
-                        new UnderReplicatedLedgersChangedCb());
-            } catch (UnavailableException ue) {
-                LOG.error("Exception while registering for under-replicated ledgers change notification, so exiting",
-                        ue);
+                LOG.error("Exception while registering for change notification, so exiting", ue);
                 submitShutdownTask();
+                return;
             }
 
             scheduleBookieCheckTask();
@@ -595,8 +586,8 @@ public class Auditor implements AutoCloseable {
 
     private void waitIfLedgerReplicationDisabled() throws UnavailableException,
             InterruptedException {
-        ReplicationEnableCb cb = new ReplicationEnableCb();
         if (!ledgerUnderreplicationManager.isLedgerReplicationEnabled()) {
+            ReplicationEnableCb cb = new ReplicationEnableCb();
             LOG.info("LedgerReplication is disabled externally through Zookeeper, "
                     + "since DISABLE_NODE ZNode is created, so waiting untill it is enabled");
             ledgerUnderreplicationManager.notifyLedgerReplicationEnabled(cb);
