@@ -21,12 +21,19 @@ package org.apache.bookkeeper.tools.cli.commands.client;
 import static org.apache.bookkeeper.common.concurrent.FutureUtils.result;
 
 import com.beust.jcommander.Parameter;
+import com.google.common.collect.ImmutableMap;
 import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
+import java.nio.charset.StandardCharsets;
+import java.util.Arrays;
+import java.util.Random;
 import java.util.concurrent.TimeUnit;
 import lombok.Setter;
 import lombok.experimental.Accessors;
 import org.apache.bookkeeper.client.api.BookKeeper;
 import org.apache.bookkeeper.client.api.DigestType;
+import org.apache.bookkeeper.client.api.LedgerEntries;
+import org.apache.bookkeeper.client.api.LedgerEntry;
+import org.apache.bookkeeper.client.api.ReadHandle;
 import org.apache.bookkeeper.client.api.WriteHandle;
 import org.apache.bookkeeper.tools.cli.commands.client.SimpleTestCommand.Flags;
 import org.apache.bookkeeper.tools.cli.helpers.ClientCommand;
@@ -41,7 +48,7 @@ import org.slf4j.LoggerFactory;
 public class SimpleTestCommand extends ClientCommand<Flags> {
 
     private static final String NAME = "simpletest";
-    private static final String DESC = "Simple test to create a ledger and write entries to it.";
+    private static final String DESC = "Simple test to create a ledger and write entries to it, then read it.";
     private static final Logger LOG = LoggerFactory.getLogger(SimpleTestCommand.class);
 
     /**
@@ -74,18 +81,23 @@ public class SimpleTestCommand extends ClientCommand<Flags> {
     }
 
     @Override
-    @SuppressFBWarnings("RCN_REDUNDANT_NULLCHECK_WOULD_HAVE_BEEN_A_NPE")
+    @SuppressFBWarnings({"RCN_REDUNDANT_NULLCHECK_WOULD_HAVE_BEEN_A_NPE", "DMI_RANDOM_USED_ONLY_ONCE"})
     protected void run(BookKeeper bk, Flags flags) throws Exception {
         byte[] data = new byte[100]; // test data
-
-        try (WriteHandle wh = result(bk.newCreateLedgerOp()
-            .withEnsembleSize(flags.ensembleSize)
-            .withWriteQuorumSize(flags.writeQuorumSize)
-            .withAckQuorumSize(flags.ackQuorumSize)
-            .withDigestType(DigestType.CRC32C)
-            .withPassword(new byte[0])
-            .execute())) {
-
+        Random random = new Random(0);
+        for (int i = 0; i < data.length; i++) {
+            data[i] = (byte) (random.nextInt(26) + 65);
+        }
+        WriteHandle wh = null;
+        try {
+            wh = result(bk.newCreateLedgerOp()
+                    .withEnsembleSize(flags.ensembleSize)
+                    .withWriteQuorumSize(flags.writeQuorumSize)
+                    .withAckQuorumSize(flags.ackQuorumSize)
+                    .withDigestType(DigestType.CRC32C)
+                    .withCustomMetadata(ImmutableMap.of("Bookie", NAME.getBytes(StandardCharsets.UTF_8)))
+                    .withPassword(new byte[0])
+                    .execute());
             LOG.info("Ledger ID: {}", wh.getId());
             long lastReport = System.nanoTime();
             for (int i = 0; i < flags.numEntries; i++) {
@@ -97,6 +109,21 @@ public class SimpleTestCommand extends ClientCommand<Flags> {
                 }
             }
             LOG.info("{} entries written to ledger {}", flags.numEntries, wh.getId());
+
+            try (ReadHandle rh = result(bk.newOpenLedgerOp().withLedgerId(wh.getId()).withDigestType(DigestType.CRC32C)
+                    .withPassword(new byte[0]).execute())) {
+                LedgerEntries ledgerEntries = rh.read(0, flags.numEntries);
+                for (LedgerEntry ledgerEntry : ledgerEntries) {
+                    if (!Arrays.equals(ledgerEntry.getEntryBytes(), data)) {
+                        LOG.error("Read test failed, the reading data is not equals writing data.");
+                    }
+                }
+            }
+        } finally {
+            if (wh != null) {
+                wh.close();
+                result(bk.newDeleteLedgerOp().withLedgerId(wh.getId()).execute());
+            }
         }
     }
 }
